package com.hw.wms.mapper;

import java.util.List;
import com.hw.wms.domain.WmsInventoryCheckDetail;
import org.apache.ibatis.annotations.Param;

/**
 * 盘点记录明细Mapper接口
 *
 * @author xins
 * @date 2024-03-29
 */
public interface WmsInventoryCheckDetailMapper
{
    /**
     * 查询盘点记录明细
     *
     * @param inventoryCheckDetailId 盘点记录明细主键
     * @return 盘点记录明细
     */
    public WmsInventoryCheckDetail selectWmsInventoryCheckDetailByInventoryCheckDetailId(Long inventoryCheckDetailId);

    /**
     * 查询盘点记录明细列表
     *
     * @param wmsInventoryCheckDetail 盘点记录明细
     * @return 盘点记录明细集合
     */
    public List<WmsInventoryCheckDetail> selectWmsInventoryCheckDetailList(WmsInventoryCheckDetail wmsInventoryCheckDetail);

    /**
     * 新增盘点记录明细
     *
     * @param wmsInventoryCheckDetail 盘点记录明细
     * @return 结果
     */
    public int insertWmsInventoryCheckDetail(WmsInventoryCheckDetail wmsInventoryCheckDetail);

    /**
     * 修改盘点记录明细
     *
     * @param wmsInventoryCheckDetail 盘点记录明细
     * @return 结果
     */
    public int updateWmsInventoryCheckDetail(WmsInventoryCheckDetail wmsInventoryCheckDetail);

    /**
     * 删除盘点记录明细
     *
     * @param inventoryCheckDetailId 盘点记录明细主键
     * @return 结果
     */
    public int deleteWmsInventoryCheckDetailByInventoryCheckDetailId(Long inventoryCheckDetailId);

    /**
     * 批量删除盘点记录明细
     *
     * @param inventoryCheckDetailIds 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteWmsInventoryCheckDetailByInventoryCheckDetailIds(Long[] inventoryCheckDetailIds);

    /**
     * 根据盘点记录ID和库位编码获取盘点记录明细
     * @param inventoryCheckId
     * @param locationCode
     * @return
     */
    public int selectCountOfCheckDetailByCheckIdAndLocation(@Param("inventoryCheckId") Long inventoryCheckId,
                                                            @Param("locationCode") String locationCode);

    /**
     * 查询盘点记录明细列表,Join material
     *
     * @param wmsInventoryCheckDetail 盘点记录明细
     * @return 盘点记录明细集合
     */
    public List<WmsInventoryCheckDetail> selectWmsInventoryCheckDetailJoinList(WmsInventoryCheckDetail wmsInventoryCheckDetail);
}
